<?php

declare(strict_types=1);

namespace Dedi\SyliusClicAndPayPlugin\Payum;

use Http\Message\MessageFactory;
use Payum\Core\HttpClientInterface;
use Payum\Core\Reply\HttpPostRedirect;
use Sylius\Bundle\PayumBundle\Model\PaymentSecurityTokenInterface;
use Sylius\Component\Core\Model\OrderInterface;

class ClicAndPayApi implements ClicAndPayParams
{
    const CODE = 'CLIC_AND_PAY';

    const ACCEPTED_STATUS = 'accepted';

    const CANCELLED_STATUS = 'cancelled';

    const REFUSE_STATUS = 'refuse';

    protected HttpClientInterface $client;

    protected MessageFactory $messageFactory;

    protected array $options = [];

    public function __construct(array $options, HttpClientInterface $client, MessageFactory $messageFactory)
    {
        $this->options = $options;
        $this->client = $client;
        $this->messageFactory = $messageFactory;
    }

    public function doPayment(array $details, OrderInterface $order, PaymentSecurityTokenInterface $token): void
    {
        $now = (new \DateTime())->setTimezone(new \DateTimeZone('UTC'));
        $fields = [];
        $fields[self::VADS_ACTION_MODE] = $this->options[self::VADS_ACTION_MODE];
        $fields[self::VADS_CTX_MODE] = $this->options['sandbox'] ? 'TEST' : 'PRODUCTION';
        $fields[self::VADS_CURRENCY] = $this->options[self::VADS_CURRENCY];
        $fields[self::VADS_PAGE_ACTION] = $this->options[self::VADS_PAGE_ACTION];
        $fields[self::VADS_SITE_ID] = $this->options[self::VADS_SITE_ID];
        $fields[self::VADS_VERSION] = $this->options[self::VADS_VERSION];

        $fields[self::VADS_TRANS_DATE] = $now->format('YmdHis');
        $fields[self::VADS_TRANS_ID] = substr($order->getTokenValue(), 0, 6);
        $fields[self::VADS_AMOUT] = $order->getTotal();

        $fields[self::VADS_RETURN_MODE] = 'GET';
        $fields[self::VADS_URL_RETURN] = $token->getTargetUrl();

        if ($this->options[self::IS_MULTI_PAYMENT]) {
            $fields[self::VADS_PAYMENT_CONFIG] = sprintf(
                'MULTI:first=%d;count=%d;period=%s',
                round(((int) $this->options[self::MULTI_PAYMENT_FIRST_PERCENT] / 100) * $order->getTotal()),
                $this->options[self::MULTI_PAYMENT_COUNT],
                $this->options[self::MULTI_PAYMENT_PERIOD]
            );
        } else {
            $fields[self::VADS_PAYMENT_CONFIG] = 'SINGLE';
        }

        $fields[self::SIGNATURE] = $this->getSignature($fields);

        throw new HttpPostRedirect($this->getApiEndpoint(), $fields);
    }

    protected function getSignature($details): string
    {
        $clientKey = $this->options[self::VADS_SITE_KEY];
        $signature = [];
        foreach (self::VADS_ORDER_TO_SIGNATURE as $key) {
            if (!isset($details[$key])) {
                throw new \RuntimeException('Clic and Pay : Need ' . $key . ' parameter to create signature.');
            }

            $signature[] = $details[$key];
        }

        $signature[] = $clientKey;

        return base64_encode(hash_hmac(self::HASH, implode('+', $signature), $clientKey, true));
    }

    protected function getApiEndpoint(): string
    {
        return 'https://' . self::SERVERS[0] . '/' . self::URL_CLASSIC;
    }
}
